classdef McsCmosStaExplorerSource < handle
% Holds the contents of a STAExplorer data source
%
% Important fields:
%   STAData         -   (cell array) Holds the STAs for each channel/unit
%                       in the group. The STAs are read from the file the
%                       first time this array is accessed.
%   STAInfos        -   (cell array) Holds the metadata for each STA
%   Settings        -   (struct) Holds the settings of the STAExplorer
%
% (c) 2017 by Multi Channel Systems MCS GmbH
    properties (SetAccess = private)
        STAData = {};   % (cell array) STAs for each channel/unit
        STAInfos = {}; % (struct) metadata for each STA
        DataDimensions = 'columns x rows x samples'; % The dimension of the STA matrix
        Label           % (string) The name of the STAExplorer
        DataSubType
        DataUnit % (string) The data unit for the STA data
        DataType = 'double' % (string) The data type for the STA data
        Info % (struct) Contains metadata for the STAExplorer
        Settings = []; % (struct) Contains settings datasets, each as a field
    end
    
    properties (Access = private)
        DataLoaded = false;
        Internal = false;
        StructInfo
        FileName
    end
    
    methods
        function str = McsCmosStaExplorerSource(filename, strStruct, varargin)
        % Constructs a McsCmosStaExplorerSource object
        %
        % function str = McsCmosStaExplorerSource(filename, strStruct, varargin)
        %
        % Input:
        %   filename        -   (string) The HDF5 file name
        %   strStruct       -   (struct) The HDF5 tree structure of the
        %                       STAExplorer group generated by the h5info
        %                       command
        
            if exist('h5info')
                mode = 'h5';
            else
                mode = 'hdf5';
            end
            
            str.Label = McsHDF5.McsH5Helper.GetFromAttributes(strStruct, 'ID.Instance', mode);
            str.DataSubType = McsHDF5.McsH5Helper.GetFromAttributes(strStruct, 'SubType', mode);
            str.Info = McsHDF5.McsH5Helper.ReadInfoFromAttributes(strStruct, mode);
            
            str.Settings = McsHDF5.McsCmosStaExplorerSource.ReadSettingsDatasets(filename, strStruct, mode);
            str.STAInfos = McsHDF5.McsCmosStaExplorerSource.ReadStaInfos(strStruct, mode);
            if isempty(str.DataUnit) % only for the file version without units at the STA datasets
                str.DataUnit = repmat({'nV'}, size(str.STAInfos, 1), size(str.STAInfos, 2));
            end
            str.StructInfo = strStruct;
            str.FileName = filename;
        end
        
        function data = get.STAData(str)
        % Accessor function for the STAData field.
        %
        % function data = get.STAData(str)
        %
        % Will read the STA data from file the first time this field is
        % accessed.
            if exist('h5info')
                mode = 'h5';
            else
                mode = 'hdf5';
            end
            
            if ~str.Internal && ~str.DataLoaded
                str.Internal = true;
                fprintf('Reading STA data...')
                try 
                    for stai = 1:length(str.STAInfos)
                        if strcmp(mode, 'h5')
                            sta = h5read(str.FileName, [str.StructInfo.Name '/' str.STAInfos{stai}.IDInstance]);
                        else
                            sta = hdf5read(str.FileName, [str.StructInfo.Name '/' str.STAInfos{stai}.IDInstance]);
                        end
                        str.STAData = [str.STAData {sta}];
                    end
                end
                fprintf('done!\n');
                str.DataLoaded = true;
                str.Internal = false;
            end
            data = str.STAData;
        end
        
        function s = disp(str)
            s = 'McsCmosStaExplorerSource object\n\n';
            s = [s 'Properties:\n'];
            s = [s '\tLabel:\t\t\t ' strtrim(str.Label) '\n'];
            s = [s '\tData Type:\t\t ' str.DataSubType '\n'];
            s = [s '\tSTAs Loaded:\t\t\t '];
            if str.DataLoaded
                s = [s 'true\n'];
            else
                s = [s 'false\n'];
            end
            s = [s '\n'];
            
            s = [s 'Available Fields:\n'];
            s = [s '\tSTAData:\t\t\t\t '];
            if str.DataLoaded
                s = [s '{' num2str(size(str.STAData, 1)) 'x' num2str(size(str.STAData, 2)) ' cell}'];
            else
                s = [s 'not loaded'];
            end
            s = [s '\n'];
            s = [s '\tSTAInfos:\t\t\t\t {' num2str(size(str.STAInfos, 1)) 'x' num2str(size(str.STAInfos, 2)) ' cell}'];
            s = [s '\n'];
            s = [s '\tDataDimensions:\t\t\t ' str.DataDimensions];
            s = [s '\n'];
            s = [s '\tDataUnit:\t\t\t\t {' num2str(size(str.DataUnit, 1)) 'x' num2str(size(str.DataUnit, 2)) ' cell}'];
            s = [s '\n'];
            s = [s '\tDataType:\t\t\t\t ' str.DataType];
            s = [s '\n'];
            s = [s '\tLabel:\t\t\t\t\t ' str.Label];
            s = [s '\n'];
            s = [s '\tDataSubType:\t\t\t ' str.DataSubType];
            s = [s '\n'];
            s = [s '\tInfo:\t\t\t\t\t [1x1 struct]'];
            s = [s '\n'];
            if ~isempty(str.Settings)
                s = [s '\tSettings:\t\t\t\t [1x1 struct]'];
                s = [s '\n'];
            end
            s = [s '\n'];
            fprintf(s);
        end
        
        function out_str = readPartialSTAData(str,cfg)
        % Read a subset of STAs from the data source
        %
        % function out_str = readPartialSTAData(str,cfg)
        %
        % Reads a subset of the STAs from the HDF5 file and returns the
        % McsCmosStaExplorerSource object containing only the specific
        % STAs. Useful, if the data has not yet been read from the file
        % and the user is only interested in specific STAs.
        %
        % Input:
        %   str         -   The McsCmosStaExplorerSource object
        %
        %   cfg         -   Either empty (for default parameters) or a
        %                   structure with (some of) the following fields:
        %                   'sta': (1 x n) Array of indexes into the
        %                   STAInfos array. If empty, all STAs are loaded
        %
        % Output:
        %   out_str     -   The McsCmosStaExplorerSource object containing
        %                   the requested STAs
            if exist('h5info')
                mode = 'h5';
            else
                mode = 'hdf5';
            end
            
            defaultStas = 1:length(str.STAInfos);
            [cfg, isDefault] = McsHDF5.checkParameter(cfg, 'sta', defaultStas);
            
            % read metadata
            tmpStruct = str.StructInfo;
            tmpcfg = [];
            tmpcfg.dataType = str.DataType;
            out_str = McsHDF5.McsCmosStaExplorerSource(str.FileName, tmpStruct, tmpcfg);
            
            out_str.Internal = true;
            out_str.STAInfos = str.STAInfos(cfg.sta);
            if str.DataLoaded
                out_str.STAData = str.STAData(cfg.sta);
            else
                % read data segment
                fprintf('Reading partial STA data...');
                out_str.STAData = cell(1, length(out_str.STAInfos));
                for stai = 1:length(out_str.STAInfos)
                    if strcmp(mode, 'h5')
                        out_str.STAData{stai} = h5read(out_str.FileName, [out_str.StructInfo.Name '/' out_str.STAInfos{stai}.IDInstance]);
                    else
                        out_str.STAData{stai} = hdf5read(out_str.FileName, [out_str.StructInfo.Name '/' out_str.STAInfos{stai}.IDInstance]);
                    end
                    out_str.STAData{stai} = permute(out_str.STAData{stai},[2 1 3]);
                end
                fprintf('done!\n');
            end
            out_str.DataLoaded = true;
            out_str.DataType = str.DataType;
            out_str.Internal = false;
            out_str.DataUnit = str.DataUnit(cfg.sta);
        end
        
    end
    
    methods (Static)
        mouseSingleSensor(src, evt)
        
        mouseHandlerHeatMap(src, evt)
        
        fig = plotSingleSensor(parentFIG,UOI,coordinates)
        
        plotHeatMap(fig, coordinates, heat, sensorDimension, SourceIDs)
    end
    
    methods (Static, Access=private)
        function set = ReadSettingsDatasets(filename, strStruct, mode)
            settingsTypes = {'a95db4a1-d124-4c52-8889-2264fcdb489b',...
                '935a1aa6-4082-482e-9d4d-1ad60d1b1680',...
                '44b29fba-ec5c-48b5-8e0e-02ad9b9ac83a',...
                'de316ac6-ad66-4d78-acc4-e3f29bd40991'};
            
            set = McsHDF5.McsH5Helper.ReadDatasetsToStruct(filename, strStruct, mode, settingsTypes);
        end
        
        function staInfos = ReadStaInfos(strStruct, mode)
            staType = '442b7514-fe3a-4c66-8ae9-4f249ef48f2f';
            staInfos = {};
            for di = 1:length(strStruct.Datasets)
                type = McsHDF5.McsH5Helper.GetFromAttributes(strStruct.Datasets(di), 'ID.TypeID', mode);
                info = [];
                if strcmpi(type, staType)
                    for ai = 1:length(strStruct.Datasets(di).Attributes)
                        [name, value] = McsHDF5.McsH5Helper.AttributeNameValueForStruct(strStruct.Datasets(di).Attributes(ai), mode);
                        info.(name) = value;
                    end
                    staInfos = [staInfos {info}];
                end
            end
        end
    end
end